#ifndef ANTWORTAUSWERTER_H
#define ANTWORTAUSWERTER_H
#include "ClientAnfrage.h"
#include <boost/asio.hpp>
#include <cstdlib>
#include <cstring>
#include <fstream>
#include <string>

class AntwortAuswerter {
public:
  AntwortAuswerter(ClientAnfrage& r)
   : clientAnfrage(r), socket(r.getSocket()), dateiname(r.getDateiname()),
   header("noch undefiniert"), status(-1)
  { }

  void receive()
  {
    if (!clientAnfrage.istGesendet()) {
      clientAnfrage.send();
    }
    std::ofstream ausgabe;                   // zur Speicherung des Contents
    ausgabe.open(dateiname, std::ios::binary);
    int anzahl{0};
    constexpr auto puffergroesse{4096};
    char buf[puffergroesse];
    do {
      boost::system::error_code error;
      anzahl = socket.read_some(boost::asio::buffer(buf, puffergroesse), error);
      if (error == boost::asio::error::eof) {
        break;                               // kein Fehler, normales Ende
      }
      else {                                 // Puffer auswerten
        if (status == -1) {
          std::string tmp(buf, anzahl);
          auto headerEnde = tmp.find("\r\n\r\n");
          header = tmp.substr(0, headerEnde);
          int startInhalt = headerEnde + 4;  // CRLF überspringen
          // restliche Bytes wegschreiben
          auto rest = static_cast<int>(anzahl) - startInhalt;
          if (rest > 0) {
            ausgabe.write(buf + startInhalt, rest);
          }
          status = atoi(buf + header.find(' ') + 1);
        }
        else {
          ausgabe.write(buf, anzahl);
        }
      }
    } while (anzahl > 0);
    ausgabe.close();
  }
  auto getStatus() const { return status; }
  const std::string& getHeader() const { return header; }

private:
  ClientAnfrage& clientAnfrage;
  boost::asio::ip::tcp::socket& socket;
  const std::string& dateiname;
  std::string header;
  int status;
};
#endif
