#include <cstring>
#include <fstream>
#include <iostream>
using namespace std;

int main(int argc, char* args[])
{
  if (argc != 2) {
    cout << "Analyse der Codierung ISO8859/UTF8 auf Basis von Umlaute\n"
            "Annahme: UTF8-Zeichen mit max. zwei Bytes. Gebrauch: "
         << args[0] << " Dateiname\n";
    return 0;
  }
  ifstream quelle(args[1], ios::binary | ios::in);
  if (!quelle) {
    cerr << args[1] << " kann nicht geöffnet werden!\n";
    return -1;
  }
  char ch;
  unsigned char c;
  bool utf8Vorhanden{false};
  bool iso8859Vorhanden{false};
  bool nichtdruckbaresASCII{false};
  bool andere2bytesequenz{false};
  int fehler{0};
  int zeile{1};
  int spalte{1};
  while (quelle.get(ch)) {
    c = static_cast<unsigned char>(ch);
    ++spalte;
    if (quelle) {
      switch (c) {
      case 0x0a:          // neue Zeile
        ++zeile;
        spalte = 1;
        break;
        // ISO 8859
      case 0xc4:          // Ae
      case 0xd6:          // Oe
      case 0xdc:          // Ue
      case 0xe4:          // ae
      case 0xf6:          // oe
      case 0xfc:          // ue
      case 0xdf:          // sz
      case 0xa4:          // Euro nur bei ISO8859-15
        iso8859Vorhanden = true;
        break;
      case 0xc3:          // Alle Umlaute und ß sind in UTF-8 2-Byte-Zeichen,
        // die mit 0xc3 beginnen
        quelle.get(ch);          // 2. Byte ignorieren
        utf8Vorhanden = true;
        break;
      default:
        if (c < 0x20) {
          if (c < 0x09 || c > 0x0d) {
            nichtdruckbaresASCII = true;
          }
        }
        else if (c >= 0xc2 && c <= 0xdf) {
          andere2bytesequenz = true;
          quelle.get(ch);          // 2. Byte ignorieren
          cout << args[1] << ": Zeile " << zeile << "    Spalte " << spalte 
               << "  Zeichen " << hex
               << "0x" << static_cast<unsigned int>(c);
          c = ch;
          cout << static_cast<unsigned int>(c) << dec 
               << " (2-Byte-UTF8-Sequenz, kein Umlaut)\n";
        }
        else if (c > 0xdf) {
          cout << args[1] << ": Zeile " << zeile << "    Spalte " << spalte 
               << "  Zeichen " << hex
               << "0x" << static_cast<unsigned int>(c) << dec << '\n';
          ++fehler;
        }
      }
    }
    if(fehler > 8) {
        cout << "Zu viele falsche Bytes. Abbruch!\n";
        break;
    }
  }
  if (fehler > 0) {
    cout << args[1]
         << ": Falsche Bytes oder UTF-Mehrfachbytezeichen > 2 Byte gefunden\n"
            " (Voraussetzung für dieses Programm nicht erfüllt).\n";
    return 1;
  }
  if (andere2bytesequenz && !utf8Vorhanden) {
    cout << args[1] << ": wahrscheinlich UTF-8, enthält aber keine Umlaute\n";
  }
  if (utf8Vorhanden) {
    cout << args[1] << ": UTF-8 ";
    if (andere2bytesequenz) {
      cout << "(enthält außer Umlauten weitere 2-Byte-UTF8-Sequenzen)\n";
    }
    cout <<'\n';
  }
  if (iso8859Vorhanden) {
    cout << args[1] << ": ISO-8859-1\n";
  }
  if (nichtdruckbaresASCII) {
    cout << args[1] << ": nicht druckbare Zeichen gefunden!\n";
  }
  if (utf8Vorhanden && iso8859Vorhanden) {
    cout << "Fehler: Zeichen fuer beide Codierungen gefunden!\n";
  }
  if (!utf8Vorhanden && !iso8859Vorhanden) {
    cout << args[1] << ": ASCII ohne Umlaute (ISO8859 == UTF-8)\n";
  }
}
