#ifndef SMARTPOINTER_T
#define SMARTPOINTER_T
#include "NullPointerException.h"

template <typename T> 
class SmartPointer final {                   // Vererbung ist nicht gewollt
public:
  explicit SmartPointer(T* p = nullptr);
  ~SmartPointer();
  T* operator->() const;
  T& operator*() const;
  explicit operator bool() const;
private:
  T* zeigerAufObjekt;
  void check() const;          // Prüfung auf nicht-Null
  // Ergänzung siehe Text
  void operator=(const SmartPointer&) = delete;          // Zuweisung verbieten
  // Initialisierung mit SmartPointer verbieten:
  SmartPointer(const SmartPointer&) = delete;
};

template <typename T> SmartPointer<T>::SmartPointer(T* p) : zeigerAufObjekt{p} {}

template <typename T> void SmartPointer<T>::check() const
{
  if (!zeigerAufObjekt) {
    throw NullPointerException();
  }
}

template <typename T> SmartPointer<T>::~SmartPointer() { delete zeigerAufObjekt; }

template <typename T> T* SmartPointer<T>::operator->() const
{
  check();
  return zeigerAufObjekt;
}

template <typename T> T& SmartPointer<T>::operator*() const
{
  check();
  return *zeigerAufObjekt;
}

template <typename T> SmartPointer<T>::operator bool() const { return bool(zeigerAufObjekt); }

#endif          // SMARTPTR_T

